var InitCommand = require('../../src/commands/init');
var InitBusiness = require('../../src/business/init');
var fs = require("fs");

describe("Init Command Suite", function() {
  
    beforeEach(()=>{
        cmd = new InitCommand();

        initBusiness = new InitBusiness();
    });

    afterEach(()=>{
        var files = fs.readdirSync('.').filter((file) => {
            return file.endsWith(".kw.yml");
        });
        files.map(fs.unlinkSync);
    });

    it("init invalid object type", function() {
        expect(() => { cmd.run('sbrubles') })
            .toThrow('Não foi possível inicializar o objeto [sbrubles]: Tipo inválido')
    });

    it("init datasource", function() {
        spyOn(cmd.initBusiness, 'dump');
        spyOn(cmd.console, 'prompt').and.returnValue({
            then: function(_fn){
                _fn({
                    codigo: 'DS_1',
                    tipo: 'jdbc'
                })
            }
        });    

        cmd.run("datasource");

        expect(cmd.initBusiness.dump.calls.argsFor(0)[0]).toEqual('DS_1');
        expect(cmd.initBusiness.dump.calls.argsFor(0)[1]).toEqual({
            datasources: {
                DS_1: {
                    type: 'jdbc',
                    url: null,
                    user: null,
                    password: null,
                    dialect: null

                }
            }
        });
    });


    it("init dataset", function() {
        spyOn(cmd.initBusiness, 'dump');
        spyOn(cmd.console, 'prompt').and.returnValue({
            then: function(_fn){
                _fn({
                    codigo: 'DSET_1',
                    datasource: 'DS_1',
                    name: 'DataSet One'
                })
            }
        });           

        cmd.run("dataset");

        expect(cmd.initBusiness.dump.calls.argsFor(0)[0]).toEqual('DSET_1');
        expect(cmd.initBusiness.dump.calls.argsFor(0)[1]).toEqual({
            datasets: {
                DSET_1: {
                    name: 'DataSet One',
                    description: 'DataSet One',
                    datasource: 'DS_1',
                    replicate: true,
                    columns: {
                        ID: ['number', 'Identificador'],
                        LAST_UPDATED: ['date', "Data de ultima atualizacao"]
                    }
                }
            }
        });
    });   

    it("init chart", function() {
        spyOn(cmd.initBusiness, 'dump');
        spyOn(cmd.console, 'prompt').and.returnValue({
            then: function(_fn){
                _fn({
                    codigo: 'CHART_1',
                    dataset: 'DSET_1',
                    tipo: 'bar',
                    titulo: 'Chart One'
                })
            }
        });           

        cmd.run("chart");

        expect(cmd.initBusiness.dump.calls.argsFor(0)[0]).toEqual('CHART_1');
        expect(cmd.initBusiness.dump.calls.argsFor(0)[1]).toEqual({
            charts: {
                CHART_1: {
                    data: {
                        dataset: 'DSET_1',
                        mapping: {
                            X: 'COL_1',
                            Y: 'COL_2'
                        }
                    },
                    options: {
                        type: 'bar',
                        title: 'Chart One',
                        margin: '50 50 50 50'
                    }
                }
            }
        });
    });      

    it("init report", function() {
        spyOn(cmd.initBusiness, 'dump');
        spyOn(cmd.console, 'prompt').and.returnValue({
            then: function(_fn){
                _fn({
                    codigo: 'REPORT_1',
                    titulo: 'Report One'
                })
            }
        });           

        cmd.run("report");

        expect(cmd.initBusiness.dump.calls.argsFor(0)[0]).toEqual('REPORT_1');
        expect(cmd.initBusiness.dump.calls.argsFor(0)[1]).toEqual({
            reports: {
                REPORT_1: {
                    title: 'Report One',
                    navigation: ['CHART_1', 'CHART_2'],
                    filters: ['COL_1', 'COL_2']
                }
            }
        });
    });      


});