var PushCommand = require('../../src/commands/push');
var EnvBusiness = require("../../src/business/environment");
var PushBusiness = require("../../src/business/push");
var InitBusiness = require("../../src/business/init");
var TestObjects = require("../testObjects");
var fs = require("fs");

function _spyOnPost(cmd){
    spyOn(cmd.pushBusiness.api, 'post').and.returnValue(new Promise((resolve => {
      resolve();
    })));
}

function _spyOnDoPost(cmd){
    spyOn(cmd.pushBusiness.api, '_doPost').and.returnValue(new Promise((resolve => {
      resolve();
    })));
}

function _run(cmd, thenFn){
    cmd.run("prod").then(thenFn).catch(function(e){
      
    });
}

describe("Push Command Suite", function() {

  beforeEach(()=>{
    cmd = new PushCommand();

    pushBusiness = new PushBusiness();

    envBusiness =  new EnvBusiness();
    envBusiness.add("prod", "http://localhost:8080/insight");

    initBusiness = new InitBusiness();
  });

  afterEach(()=>{
    var files = fs.readdirSync('.').filter((file) => {
      return file.endsWith(".kw.yml");
    });
    files.map(fs.unlinkSync);
  });


  it("push to non existing environment", () => {
    cmd.run("sbrubles")
    .then(() => {
    })
    .catch((e) => {
      expect(e).toBe("Environment not found");      
    });
  });

  it("simple push datasource", function(){
    let ds1 = {
      datasources: {
          DS_1: {
            type: "jdbc",
            url: "",
            user: "",
            pass: "",
            dialect: ""
          }
        }
    }

    let ds2 = {
      datasources: {
          DS_2: {
            type: "jdbc",
            url: "",
            user: "",
            pass: "",
            dialect: ""
          }
        }
    }

    initBusiness.init("ds", ds1);
    initBusiness.init("ds2", ds2);

    _spyOnPost(cmd);

    _run(cmd, () => {
      expect(cmd.pushBusiness.api.post.calls.argsFor(0)[1]).toEqual([ds1, ds2]);
    });

  });



  it("simple push datasource", function(){

    let ds1 = {
      datasources: {
          DS_1: {
            type: "jdbc",
            url: "foo@localhost:5432/db",
            user: "foo",
            password: "bar",
            dialect: "postgres"
          }
        }
    }

    initBusiness.init("ds", ds1);

    _spyOnDoPost(cmd);

    _run(cmd, () => {
      expect(cmd.pushBusiness.api._doPost.calls.argsFor(0)[1]).toEqual({
        datasources: [{
            code: "DS_1",
            type: "jdbc",
            options: {
                url: "foo@localhost:5432/db",
                user: "foo",
                password: "bar",
                dialect: "postgres"
            }
        }]
      });
    });

  });

  it("push datasource and dataset", function(){
    initBusiness.init("ds", TestObjects.DS_1);
    initBusiness.init("dset_1", TestObjects.DSET_1);

    spyOn(pushBusiness.api, '_doPost');
    
    pushBusiness.push("prod");

    expect(pushBusiness.api._doPost.calls.argsFor(0)[1]).toEqual({
      datasources: [{
          code: "DS_1",
          type: "jdbc",
          options: {
              url: "foo@localhost:5432/db",
              user: "foo",
              password: "bar",
              dialect: "postgres"
          }
      }],
      datasets: [{
        codigo: "DSET_1",
        datasource: "DS_1",
        name: "Datasource 1",
        description: "Datasource 1",
        query: "select * from users",
        columns: [
          {name: "ID", type: 2, precision: 19, scale: 2, description: "Identificador"},
          {name: "NAME", type: 12, length: 255, description: "Nome"},
          {name: "BIRTHDATE", type: 93, description: "Data de Nascimento"},
          {name: "SCORE", type: 2, scale: 0, precision: 19, description: "Pontuacao"},
          {name: "STATEMENT", type: 12, length: 255, description: "Declaracao"}
        ]
      }]
    }); 
  });

  it("push grafico de barras completo", function(){
    let chart1 = {
      charts: {
          CH_1: {
            data: {
              dataset: "DSET_1",
              mapping: {
                X: "COL_1",
                Y: "COL_2"
              }
            },
            options: {
              type: "bar",
              title: "Titulo",
              margin: "50 60 70 80",
              legend: {
                  vertical: "Legenda Vertical",
                  verticalPosition: "Posicao da Legenda Vertical",
                  horizontal: "Legenda Horizontal",
                  horizontalPosition: "Posicao da Legenda Horizontal"
              },
              colorMapping:[{RJ: "red"}, {RJ: "green"}],
              cutMark:{
                  value: 100,
                  description: "Estourouu!!",
                  colorAbove: "red",
                  colorBelow: "green"
              },
              format: {COL_1: "money", COL_2: "date"},
              aggregation: {COL_1: "sum"}
            }
          }
        }
    }

    initBusiness.init("ch_1", chart1);

    _spyOnDoPost(cmd);

    _run(cmd, () => {
      expect(cmd.pushBusiness.api._doPost.calls.argsFor(0)[1]).toEqual({
        charts: [{
          codigo: "CH_1",
          name: "Titulo",
          chartType: "bar",
          dataset: "DSET_1",
          options: {
            legend: {
              horizontal: "Legenda Horizontal",
              horizontalPosition: "Posicao da Legenda Horizontal",
              vertical: "Legenda Vertical",
              verticalPosition: "Posicao da Legenda Vertical"
            },
            margin: {
              top: '50',
              right: '60',
              bottom: '70',
              left: '80'
            },
            cutMark: {
              value: 100,
              label: "Estourouu!!",
              aboveColor: "red",
              belowColor: "green"
            }
          },
          dimensions: [{
            mappedColumns: [{
              name: "COL_1",
              alias: "COL_1",
              options: {
                formatter: "money"
              }
            }],
            name: "X",
            aggregation: "sum"
          }, {
            mappedColumns: [{
              name: "COL_2",
              alias: "COL_2",
              options: {
                formatter: "date"
              }
            }],
            name: "Y"
          }]
        }]
      }); 
    });   
  });



  it("push grafico de barras minimo", function(){
    let chart1 = {
      charts: {
        CH_2: {
          data: {
            dataset: "DSET_1",
            mapping:{
              X: "COL_3",
              Y: "COL_4"
            }
          },
          options:{
            title: "Chart 2",
            type: "bar"
          }
        }
      }
    }

    initBusiness.init("ch_1", chart1);

    _spyOnDoPost(cmd);

    _run(cmd, () => {
      expect(cmd.pushBusiness.api._doPost.calls.argsFor(0)[1]).toEqual({
        charts: [{
          codigo: "CH_2",
          name: "Chart 2",
          chartType: "bar",
          dataset: "DSET_1",
          dimensions: [{
            mappedColumns: [{
              name: "COL_3",
              alias: "COL_3"
            }],
            name: "X"
          }, {
            mappedColumns: [{
              name: "COL_4",
              alias: "COL_4"
            }],
            name: "Y"
          }]   
        }]
      });
    });

  });

  it("push report", function(){
    initBusiness.init("rp_1", {
      reports: {
        RP_1: {
          title: "RP Title",
          dataset: "DSET_1",
          navigation: ["CHA_1", "CHA_2"]
        }
      }
    });

    _spyOnDoPost(cmd);

    _run(cmd, () => {
      expect(cmd.pushBusiness.api._doPost.calls.argsFor(0)[1]).toEqual({
        reports: [{
          codigo: "RP_1",
          title: "RP Title",
          dataset: "DSET_1",
          items: [{
            chart: "CHA_1",
            description: " ",
            position: 0,
          }, {
            chart: "CHA_2",
            description: " ",
            position: 1,
          }]
        }]
      });
    });

  });

  it("push", function(){
    initBusiness.init("ch_1", {
      datasources: {
        DSRC_1: {
          type: "jdbc",
          url: "jdbc:postgresql://0.0.0.0:5432/my-kw-db",
          user: "kw-db",
          pass: "kw-db"
        }
      },
      datasets: {
        DSET_1: {
          datasource: "DSRC_1",
          query: "select * from pagamentos"
        }
      },
      charts: {
        CH_2: {
          data: {
            dataset: "DSET_1",
            mapping:{
              X: "ESTAB",
              Y: "VALOR"
            }
          },
          options:{
            title: "Chart 2",
            type: "bar"
          }
        }
      },
      reports: {
        RP_1: {
          title: "RP 1",
          dataset: "DSET_1",
          navigation: ["CH_2"]
        }
      }
    });

    _run(cmd, () => {

    });

  });


  it("push 2 graficos", function(){
    let chart1 = {
      charts: {
        CH_1: {
          data: {
            dataset: "DSET_1",
            mapping:{
              X: "COL_3",
              Y: "COL_4"
            }
          },
          options:{
            title: "Chart 1",
            type: "bar"
          }
        }
      }
    }

    let chart2 = {
      charts: {
        CH_2: {
          data: {
            dataset: "DSET_1",
            mapping:{
              X: "COL_3",
              Y: "COL_4"
            }
          },
          options:{
            title: "Chart 2",
            type: "bar"
          }
        }
      }
    }    

    initBusiness.init("ch_1", chart1);
    initBusiness.init("ch_2", chart2);

    _spyOnDoPost(cmd);

    _run(cmd, () => {
      expect(cmd.pushBusiness.api._doPost.calls.argsFor(0)[1]).toEqual({
        charts: [{
          codigo: "CH_1",
          name: "Chart 1",
          chartType: "bar",
          dataset: "DSET_1",
          dimensions: [{
            mappedColumns: [{
              name: "COL_3",
              alias: "COL_3"
            }],
            name: "X"
          }, {
            mappedColumns: [{
              name: "COL_4",
              alias: "COL_4"
            }],
            name: "Y"
          }]   
        },
        {
          codigo: "CH_2",
          name: "Chart 2",
          chartType: "bar",
          dataset: "DSET_1",
          dimensions: [{
            mappedColumns: [{
              name: "COL_3",
              alias: "COL_3"
            }],
            name: "X"
          }, {
            mappedColumns: [{
              name: "COL_4",
              alias: "COL_4"
            }],
            name: "Y"
          }]   
        }]
      });
    });

  });  


});