var yaml = require('js-yaml');
var BaseBusiness = require('./base');


class InitBusiness extends BaseBusiness {

    init(prefix, obj){
        return this.dump(prefix, obj);
    }

    initDatasource(prefix, params){
        var datasources = {};
        datasources[params.codigo] = {
            type: params.tipo,
            url: null,
            user: null,
            password: null,
            dialect: null
        }

        return this.init(prefix, {datasources: datasources});
    }

    initDataset(prefix, params){
        var datasets = {};
        datasets[params.codigo] = {
            name: params.name,
            description: params.name,
            datasource: params.datasource,
            replicate: true,
            columns: {
                ID: ['number', 'Identificador'],
                LAST_UPDATED: ['date', "Data de ultima atualizacao"]
            }
        }

        return this.init(prefix, {datasets: datasets});
    }

    initChart(prefix, params){
        var charts = {};
        charts[params.codigo] = {
            data: {
                dataset: params.dataset,
                mapping: {
                    X: 'COL_1',
                    Y: 'COL_2'
                }
            },
            options: {
                type: params.tipo,
                title: params.titulo,
                margin: '50 50 50 50'
            }
        }
        
        return this.init(prefix, {charts: charts});
    }

    initReport(prefix, params){
        var reports = {};
        reports[params.codigo] = {
            title: params.titulo,
            navigation: ['CHART_1', 'CHART_2'],
            filters: ['COL_1', 'COL_2']
        }

        return this.init(prefix, {reports: reports});
    }

    initDashboard(prefix, params){
        var dashboard = {};
        dashboard[params.codigo] = {
            title: params.titulo,
            grid: [['CHART_1', 'CHART_2'],['CHART_3']],
            filters: ['COL_1', 'COL_2']
        }

        return this.init(prefix, {dashboards: dashboard});
    }

}

module.exports = InitBusiness;