#!/usr/bin/env node

var program = require('commander');
var InitCommand = require('./commands/init');
var EnvCommand = require('./commands/env');
var PushCommand = require('./commands/push');
var PkgCommand = require('./commands/pkg');
var Console = require('./commands/console');
var axios = require('axios');

var _get = require('lodash/get');
var _map = require('lodash/map');


const logYamlException = (console, error) => {
  console.logError(error.reason);
  console.logDim(error.mark.buffer);
  console.logError(error.message);
}

const run = (command) => {
  return function(args){
    let console = new Console();

    let runPromise = command.run(args);

    Promise.all([runPromise])
    .then(function(messages){
      messages.map(console.logSuccess);
      process.exit(0);
    })
    .catch(function(error){
      console.logErrorBanner();

      if(_get(error, 'name') === 'YAMLException'){
        logYamlException(console, error);
      } else {
        console.log(error);
      }

      process.exit(1);
    });

  }
}

program
  .command('init <type>')
  .description('Cria um template para definicão de um objeto do tipo informado. Os tipos suportados são: Chart e Report')
  .action(run(new InitCommand()));


program
  .command('env')
  .option('-a, --add [name]', 'Adds a new Environment')
  .option('-l, --list ', 'Lists all Environments')
  .option('-d, --delete [name]', 'Delete an Environment')
  .description('Configura um ambiente do Kowalski')
  .action(run(new EnvCommand()));


program
  .command('push <env>')
  .description('Cria ou atualiza todos os objetos declarados no ambiente selecionado')
  .action(run(new PushCommand()));


program
  .command('pkg')
  .description('Cria um pacote para ser distribuido.')
  .action(run(new PkgCommand()));

program.parse(process.argv);
